@@ -1,5 +1,5 @@
 /***************************************************************
-* Unit:    rasters           release 0.33                      *
+* Unit:    rasters           release 0.37                      *
 * Purpose: Test of functionality of raster container library.  *
 * Licency: GPL (only)                                          *
 * Copyright: (c) 1998-2025 Jaroslav Fojtik                     *
@@ -34,6 +34,8 @@ const char* const List1[] = {String1,String2}; */
 void Benchmark(void);
 void Benchmark2(void);
 void Benchmark3(void);
+void Benchmark4(void);
+void Benchmark5(void);
 void B2(void);
 
 FILE *TempF;
@@ -224,30 +226,51 @@ int m;
 
  puts("");
  puts("<<<rasters>>> (c)1998-2025 Jaroslav Fojtik");
- printf("         This program tests the raster library, release %d.%d\n",
-	RASTER_VERSION>>8, RASTER_VERSION&0xFF);
+ printf("         This program tests the raster library, release %d.%d %ubits\n",
+	RASTER_VERSION>>8, RASTER_VERSION&0xFF, (unsigned)sizeof(void*)*8);
 #ifdef LO_ENDIAN
  puts("***LO_ENDIAN defined***");
 #endif
 #ifdef HI_ENDIAN
  puts("***HI_ENDIAN defined***");
 #endif
+#ifdef USE_MMX
+puts("***USE_MMX defined***");
+#endif
+#ifdef USE_SSE
+puts("***USE_SSE defined***");
+#endif
+#ifdef NATIVE_ACCESS
+ puts("***NATIVE access***");
+#endif
 
  //B2();
 
  if(argc>1)
-   {
+ {
    for(j=1; j<argc; j++)
+   {
+     if(argv[j][0]!=0 && argv[j][1]!=0 && (argv[j][0]=='-' || argv[j][0]=='/'))
      {
-     if(!strcmp(argv[j],"-bench3"))    Ops=8;
-     if(!strcmp(argv[j],"-bench2"))    Ops=4;
-     if(!strcmp(argv[j],"-bench"))     Ops|=3;
-     if(!strcmp(argv[j],"-demo"))      Ops|=4;
-     if(!strcmp(argv[j],"-test"))      Ops|=1;
-     if(!strcmp(argv[j],"-benchonly")) Ops|=2;
-     if(!strcmp(argv[j],"-pause"))     Pause=true;
+       if(!strcmp(argv[j]+1,"bench5"))    Ops=16;
+       if(!strcmp(argv[j]+1,"bench4"))    Ops=12;
+       if(!strcmp(argv[j]+1,"bench3"))    Ops=8;
+       if(!strcmp(argv[j]+1,"bench2"))    Ops=4;
+       if(!strcmp(argv[j]+1,"bench"))     Ops|=3;
+       if(!strcmp(argv[j]+1,"demo"))      Ops|=4;
+       if(!strcmp(argv[j]+1,"test"))      Ops|=1;
+       if(!strcmp(argv[j]+1,"benchonly")) Ops|=2;
+       if(!strcmp(argv[j]+1,"pause"))     Pause=true;
+       if(!strcmp(argv[j]+1,"help"))
+       {
+         printf("Command line parameters:\n-bench -bench2 -bench3 -benchonly -demo -pause -help -test\n"
+                "\t-bench2\tAnalyses flip with unalligned blob\n."
+                "\t-bench3\tAnalyses of peel and join operations\n.");
+         return 1;
+       }
      }
    }
+ }
 
  if(Ops==0) Ops=3;
 
@@ -263,7 +286,7 @@ int m;
 
 //#if 1
 
- printf(" get/set pixel 1D [");
+ printf(" get/set pixel 1D ["); fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
  {
    printf("%s%d", j==0?"":",", vec[j]);
@@ -302,7 +325,7 @@ int m;
  printf("]\n");
 
 
- printf(" get/set pixel 2D [");
+ printf(" get/set pixel 2D ["); fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
    {
    printf("%s%d", j==0?"":",", vec[j]);
@@ -338,7 +361,7 @@ int m;
  printf("]\n");
 
 #ifdef RASTER_3D
- printf(" get/set pixel 3D [");
+ printf(" get/set pixel 3D ["); fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
    {
    printf("%s%d", j==0?"":",", vec[j]);
@@ -379,7 +402,7 @@ int m;
 #endif
 
 
- printf(" get/set pixel 1D float");
+ printf(" get/set pixel 1D float"); fflush(stdout);
  for(j=0; j<sizeof(vecFlt); j++)
    {
    const unsigned ARRAY_SIZE = 1000;
@@ -446,7 +469,7 @@ int m;
  printf("]\n");
 
 
- printf(" get/set pixel 2D float [");
+ printf(" get/set pixel 2D float ["); fflush(stdout);
  for(j=0; j<sizeof(vecFlt); j++)
    {
    printf("%s%d", j==0?"":",", vecFlt[j]);
@@ -483,7 +506,7 @@ int m;
 
 
 #ifdef RASTER_3D
- printf(" get/set pixel 3D float [");
+ printf(" get/set pixel 3D float ["); fflush(stdout);
  for(j=0; j<sizeof(vecFlt); j++)
    {
    printf("%s%d", j==0?"":",", vecFlt[j]);
@@ -524,7 +547,7 @@ int m;
 #endif
 
 
- printf(" get/set whole container 1D [");
+ printf(" get/set whole container 1D ["); fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
    {
    if(vec[j] <= -32) continue;			// float capable containrs are not fully finished for this.
@@ -569,7 +592,7 @@ int m;
 	 err = RA1->GetValue1D(i);
 	 err = RA3->GetValue1D(i);
 	 printf("\nWrong value read at IDX:%d, planes: %d->%d->%d, values:%lXh;%lXh->(through)%lXh->%lXh", i,
-		vec[j], vec[k], vec[j], (long)ValueStored, (long)RA1->GetValue1D(i), (long)RA3->GetValue1D(i), (long)ValueGet);
+		vec[j], vec[k], vec[j], (unsigned long)ValueStored, (unsigned long)RA1->GetValue1D(i), (unsigned long)RA3->GetValue1D(i), (unsigned long)ValueGet);
          printf("\nShift=%d; MASK=%lX", shift, MASK);
          PrintRasters(RA1);
          PrintRasters(RA3);
@@ -592,7 +615,7 @@ int m;
  printf("]\n");
 
 
- printf(" boundary settings 1D");
+ printf(" boundary settings 1D"); fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
    {
    printf("%s%d", j==0?" [":",", vec[j]);
@@ -631,7 +654,7 @@ int m;
   printf("]\n");
 
 
-  puts(" boundary settings 2D");
+  puts(" boundary settings 2D"); fflush(stdout);
   const int BOUNDARY2D_2 = 5;
   for(i=0; i<259; i++)			// iterate Size1D
     for(j=0; j<MAX_K; j++)		// iterate bit planes
@@ -704,7 +727,7 @@ int m;
 
 
 #ifdef RASTER_3D
-   puts(" boundary settings 3D");
+   puts(" boundary settings 3D"); fflush(stdout);
    const int BOUNDARY3D_2 = 3;
    const int BOUNDARY3D_3 = 5;
    m = 0;
@@ -817,7 +840,7 @@ int m;
 #endif  // RASTER_3D
 
 
- printf(" RGB get/set pixel");
+ printf(" RGB get/set pixel"); fflush(stdout);
  for(j=0; j<sizeof(vecRGB); j++)
    {
    printf("%s%d", j==0?" [":",", vecRGB[j]);
@@ -834,7 +857,7 @@ int m;
 
    for(i=0; i<3000; i++)
      {
-     RGBval = (((i+5)%256) | (((i+6)%256)<<8) | (((i+7)%256)<<16)) & 0x00FFFFFF;
+     RGBval = (((i+5)%256) | (((i+6)%256)<<8) | (((uint32_t)(i+7)%256)<<16)) & 0x00FFFFFF;
      if(vecRGB[j]==5) RGBval &= 0x00F8FCF8;
 
      RA1->SetValue1D(i%1000, RGBval);
@@ -891,7 +914,7 @@ int m;
 
 
 	//---------------------------------------------
- printf(" RGBA get/set pixel");
+ printf(" RGBA get/set pixel"); fflush(stdout);
  for(j=0; j<sizeof(vecRGBA); j++)
    {
    printf("%s%d", j==0?" [":",", vecRGBA[j]);
@@ -908,7 +931,7 @@ int m;
 
    for(i=0; i<3000; i++)
      {
-     RGBAval = (((i+5)%256) | (((i+6)%256)<<8) | (((i+7)%256)<<16) | (((i+9)%256)<<24));
+     RGBAval = (((i+5)%256) | (((i+6)%256)<<8) | (((uint32_t)(i+7)%256)<<16) | (((uint32_t)(i+9)%256)<<24));
      RA1->SetValue1D(i%1000, RGBAval);
      if(RA1->GetValue1D(i%1000) != RGBAval)
        {
@@ -951,7 +974,7 @@ int m;
 
 
 	//---------------------------------------------
- printf(" peel/join 1bit [");
+ printf(" peel/join 1bit ["); fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
  {
    printf("%s%d", j==0?"":",", vec[j]); fflush(stdout);
@@ -1115,7 +1138,7 @@ int m;
 
 
 	//---------------------------------------------
- printf(" peel/join 8bit [");
+ printf(" peel/join 8bit ["); fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
  {
    if(labs(vec[j]) < 8) continue;
@@ -1234,7 +1257,7 @@ int m;
 
 
 	//---------------------------------------------
- printf(" flip 1D-in1D [");
+ printf(" flip 1D-in1D [");  fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
  {
    printf("%s%d", j==0?"":",", vec[j]); fflush(stdout);
@@ -1377,7 +1400,7 @@ int m;
 
 
 	//---------------------------------------------
- printf(" flip 2D-in1D [");
+ printf(" flip 2D-in1D [");  fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
    {
    printf("%s%d", j==0?"":",", vec[j]);
@@ -1414,7 +1437,7 @@ int m;
 
 
 #ifdef RASTER_3D
- printf(" flip 3D-in1D [");
+ printf(" flip 3D-in1D ["); fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
    {
    printf("%s%d", j==0?"":",", vec[j]);
@@ -1455,7 +1478,7 @@ int m;
 #endif
 
 
- printf(" flip in2D - Ras2D");
+ printf(" flip in2D - Ras2D"); fflush(stdout);
  for(i=0; i<1027; i++)
    {
    j = rand() % MAX_K;	// Planes should not be signifficant, select random.
@@ -1526,7 +1549,7 @@ int m;
  printf("\n");
 
 
- printf(" flip in3D - Ras3D");
+ printf(" flip in3D - Ras3D"); fflush(stdout);
  for(i=0; i<300; i++)
    {
    j = rand() % MAX_K;	// Planes should not be signifficant, select random.
@@ -1566,7 +1589,7 @@ int m;
 
 
 	//---------------------------------------------
- printf(" flip 1D-float");
+ printf(" flip 1D-float"); fflush(stdout);
  for(j=0; j<sizeof(vecFlt); j++)
    {
    printf("%s%d", j==0?" [":",", vecFlt[j]);
@@ -1605,7 +1628,7 @@ int m;
 
 //#endif
 	//---------------------------------------------
- printf(" flip 1D/2D [");
+ printf(" flip 1D/2D [");  fflush(stdout);
  for(j=0; j<sizeof(vec); j++)
  {
    printf("%s%d", j==0?"":",", vec[j]);
@@ -1676,6 +1699,8 @@ int m;
  /////////////////////////////////////
  if(heapcheck()>0 && err==0)
  {
+  if(Ops==16) Benchmark5();
+  if(Ops==12) Benchmark4();
   if(Ops==8) Benchmark3();
   if(Ops==4) Benchmark2();
   if(Ops & 2) Benchmark();
@@ -2137,6 +2162,7 @@ void SetGetPixel3D64(long i)
 long int j,k,l,m;
 Raster3DAbstract *RA1 = CreateRaster3D(35,33,8,64);
 
+  k = 0;
   while(i-->0)
     {
     j=i%35;
@@ -2279,6 +2305,7 @@ void BenchJoin8Bit(long i)
 }
 
 
+/////////////////////////////////////////////////////////////
 
 
 const char vecPlane[] = {0,1,2,7,8,9,12,15,16,23,24,31};
@@ -2290,11 +2317,12 @@ int i,j,k;
 
  puts(" .............. Benchmarks #3..............");
 
- printf("Peel 8 bit direction [ns/px]:\n   ");
+ printf("Peel 8 bit direction [ns/px]:  ---> pixels\n   ");
  for(i=0; i<64; i+=8)
-   {
+ {
    printf(" %7d ",i);	// print label
-   }
+ }
+ fflush(stdout);
 
  for(j=0; j<sizeof(vec); j++)
  {
@@ -2434,6 +2462,8 @@ int i,j,k;
 }
 
 
+/////////////////////////////////////////////////////////////
+
 
 void Benchmark2(void)
 {
@@ -2442,13 +2472,13 @@ int i,j;
 
  puts(" .............. Benchmarks #2..............");
 
- printf("Flip in 1D direction [ns/px]:\n   ");
+ printf("Flip in 1D direction [ns/px]:   pixels-->\n   ");
  for(i=0; i<MAX_K; i++)
-   {
+ {
    printf(" %7d ",vec[i]);	// print label
-   }
+ }
 
- printf("\n1D   ");
+ printf("\n1D   "); fflush(stdout);
  for(j=0; j<MAX_K; j++)
    {
    const unsigned FLIP_SZ_X = 14321;
@@ -2628,11 +2658,11 @@ int i,j;
 
  printf("Flip in 1D direction [us]:\n ");
  for(i=0; i<MAX_K; i++)
-   {
+ {
    printf(" %7d ",vec[i]);	// print label
-   }
+ }
 
- printf("\n1D ");
+ printf("\n1D "); fflush(stdout);
  for(j=0; j<MAX_K; j++)
    {
    const unsigned FLIP_SZ_X = 14321;
@@ -2709,7 +2739,7 @@ int i,j;
  {
    printf("%7d ",vecRGBA[i]);	// print label
  }
- printf("\nSet typed raster vector from another raster vector into RGB container [us]:");
+ printf("\nSet typed raster vector from another raster vector into RGB container [us]:"); fflush(stdout);
  for(i=0; i<sizeof(vecRGB)/sizeof(char); i++)
  {
    printf("\n[RGB%d] ",vecRGB[i]);
@@ -2778,7 +2808,7 @@ int i,j;
  {
    printf(" %7d ",vec[i]);	// print label
  }
- printf("\nGet vector from another raster vector [us]:");
+ printf("\nGet vector from another raster vector [us]:"); fflush(stdout);
  for(i=0; i<MAX_K; i++)
    {
    printf("\n[%d] ",vec[i]);
@@ -2795,7 +2825,7 @@ int i,j;
  }
  printf("\n");
 
- printf("\nSet typed raster vector from another raster vector [us]:");
+ printf("\nSet typed raster vector from another raster vector [us]:"); fflush(stdout);
  for(i=0;i<MAX_K;i++)
    {
    printf("\n[%d] ",vec[i]);
@@ -2814,7 +2844,7 @@ int i,j;
  printf("\n");
 
 
- printf("Get/Set pixel [ns]:\n ");
+ printf("Get/Set pixel [ns]:\n "); fflush(stdout);
  for(i=0;i<MAX_K;i++)
    {
    printf(" %7d ",vec[i]);	// print label
@@ -2870,3 +2900,824 @@ void B2(void)
    }
  printf("\n");
 }
+
+
+
+///////////////////////////////////////////////////////////////////////////////////
+
+const unsigned vecSz4[] = {1,3,4,7,8,15,16,31,32,1024};
+const unsigned vecSz5[] = {1024,2048};
+
+
+#ifdef USE_MMX
+
+void BenchConv4_8_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv4_8_MMX((uint8_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv4_16_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv4_16_MMX((uint16_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv4_32_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv4_32_MMX((uint32_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_4_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv8_4_MMX((uint8_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_16_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv8_16_MMX((uint16_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_32_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv8_32_MMX((uint32_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_4_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv16_4_MMX((uint8_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_8_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv16_8_MMX((uint8_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_32_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv16_32_MMX((uint32_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+#if defined(uint64_t_defined)
+void BenchConv4_64_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv4_64_MMX((uint64_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_64_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv16_64_MMX((uint64_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+#if defined(uint64_t_defined)
+void BenchConv8_64_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv8_64_MMX((uint64_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+void BenchConv32_16_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv32_16_MMX((uint16_t*)gRas1D_2->Data1D,(uint32_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+
+#if defined(uint64_t_defined)
+void BenchConv32_64_MMX(long i)
+{
+  while(i-->0)
+  {
+    Conv32_64_MMX((uint64_t*)gRas1D_2->Data1D,(uint32_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+#endif
+
+
+#ifdef USE_SSE
+
+void BenchConv4_8_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv4_8_SSE((uint8_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv4_16_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv4_16_SSE((uint16_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv4_32_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv4_32_SSE((uint32_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_4_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv8_4_SSE((uint8_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_16_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv8_16_SSE((uint16_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_32_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv8_32_SSE((uint32_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_4_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv16_4_SSE((uint8_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_8_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv16_8_SSE((uint8_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_32_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv16_32_SSE((uint32_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+#if defined(uint64_t_defined)
+void BenchConv4_64_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv4_64_SSE((uint64_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_64_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv16_64_SSE((uint64_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+#if defined(uint64_t_defined)
+void BenchConv8_64_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv8_64_SSE((uint64_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+void BenchConv32_16_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv32_16_SSE((uint16_t*)gRas1D_2->Data1D,(uint32_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+
+#if defined(uint64_t_defined)
+void BenchConv32_64_SSE(long i)
+{
+  while(i-->0)
+  {
+    Conv32_64_SSE((uint64_t*)gRas1D_2->Data1D,(uint32_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+#endif
+
+
+// !!!!!!!!!!!! UNDEFINE ACCELERATORS - needed for benchmarking !!!!!!!!!!!!
+#ifdef Conv4_8
+ #undef Conv4_8
+#endif
+#ifdef Conv4_16
+ #undef Conv4_16
+#endif
+#ifdef Conv4_32
+ #undef Conv4_32
+#endif
+#ifdef Conv4_64
+ #undef Conv4_64
+#endif
+
+#ifdef Conv8_4
+ #undef Conv8_4
+#endif
+#ifdef Conv8_16
+ #undef Conv8_16
+#endif
+#ifdef Conv8_32
+ #undef Conv8_32
+#endif
+#ifdef Conv8_64
+ #undef Conv8_64
+#endif
+
+#ifdef Conv16_4
+ #undef Conv16_4
+#endif
+#ifdef Conv16_8
+ #undef Conv16_8
+#endif
+#ifdef Conv16_32
+ #undef Conv16_32
+#endif
+#ifdef Conv16_64
+ #undef Conv16_64
+#endif
+
+#ifdef Conv32_16
+ #undef Conv32_16
+#endif
+#ifdef Conv32_64
+ #undef Conv32_64
+#endif
+
+
+void BenchConv4_8(long i)
+{
+  while(i-->0)
+  {
+    Conv4_8((uint8_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv4_16(long i)
+{
+  while(i-->0)
+  {
+    Conv4_16((uint16_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv4_32(long i)
+{
+  while(i-->0)
+  {
+    Conv4_32((uint32_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_4(long i)
+{
+  while(i-->0)
+  {
+    Conv8_4((uint8_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_16(long i)
+{
+  while(i-->0)
+  {
+    Conv8_16((uint16_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_32(long i)
+{
+  while(i-->0)
+  {
+    Conv8_32((uint32_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+#if defined(uint64_t_defined)
+void BenchConv4_64(long i)
+{
+  while(i-->0)
+  {
+    Conv4_64((uint64_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv8_64(long i)
+{
+  while(i-->0)
+  {
+    Conv8_64((uint64_t*)gRas1D_2->Data1D,(uint8_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+void BenchConv16_4(long i)
+{
+  while(i-->0)
+  {
+    Conv16_4((uint8_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_8(long i)
+{
+  while(i-->0)
+  {
+    Conv16_8((uint8_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+void BenchConv16_32(long i)
+{
+  while(i-->0)
+  {
+    Conv16_32((uint32_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+#if defined(uint64_t_defined)
+void BenchConv16_64(long i)
+{
+  while(i-->0)
+  {
+    Conv16_64((uint64_t*)gRas1D_2->Data1D,(uint16_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+void BenchConv32_16(long i)
+{
+  while(i-->0)
+  {
+    Conv32_16((uint16_t*)gRas1D_2->Data1D,(uint32_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+
+#if defined(uint64_t_defined)
+void BenchConv32_64(long i)
+{
+  while(i-->0)
+  {
+    Conv32_64((uint64_t*)gRas1D_2->Data1D,(uint32_t*)gRas1D_1->Data1D,gRas1D_1->Size1D);
+  }
+}
+#endif
+
+
+void Bench4Helper(unsigned Planes1, unsigned Planes2, void (*Bench_Tester)(long i))
+{
+float f;
+ for(unsigned i=0; i<sizeof(vecSz4)/sizeof(unsigned); i++)
+ {
+   gRas1D_1 = CreateRaster1D(vecSz4[i], Planes1);	// Test both even and odd size.
+   gRas1D_2 = CreateRaster1D(vecSz4[i], Planes2);    
+   if(gRas1D_1!=NULL && gRas1D_2!=NULL)
+   {
+     f = 1000*AutoMeasure(Bench_Tester) / vecSz4[i];
+     printf("%8.5g ",f);
+   }
+   delete(gRas1D_1); gRas1D_1=NULL;
+   delete(gRas1D_2); gRas1D_2=NULL;
+ }
+}
+
+
+void Benchmark4(void)
+{
+unsigned i;
+
+ puts(" .............. Benchmarks #4..............");
+
+ printf("Conversions with acceleration [ns/px]:  ---> pixels\n   ");
+ for(i=0; i<sizeof(vecSz4)/sizeof(unsigned); i++)
+ {
+   printf(" %7d ",vecSz4[i]);	// print label
+ }
+ fflush(stdout);
+
+
+#ifdef USE_MMX
+ printf("\n4_8MMX");
+ Bench4Helper(4,8,BenchConv4_8_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n4_8SSE");
+ Bench4Helper(4,8,BenchConv4_8_SSE);
+#endif
+ printf("\n4_8   ");
+ Bench4Helper(4,8,BenchConv4_8);
+
+#ifdef USE_MMX
+ printf("\n4_16MMX");
+ Bench4Helper(4,16,BenchConv4_16_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n4_16SSE");
+ Bench4Helper(4,16,BenchConv4_16_SSE);
+#endif
+ printf("\n4_16   ");
+ Bench4Helper(4,16,BenchConv4_16);
+
+#ifdef USE_MMX
+ printf("\n4_32MMX");
+ Bench4Helper(4,32,BenchConv4_32_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n4_32SSE");
+ Bench4Helper(4,32,BenchConv4_32_SSE);
+#endif
+ printf("\n4_32   ");
+ Bench4Helper(4,32,BenchConv4_32);
+
+#if defined(uint64_t_defined)
+#ifdef USE_MMX
+ printf("\n4_64MMX");
+ Bench4Helper(4,64,BenchConv4_64_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n4_64SSE");
+ Bench4Helper(4,64,BenchConv4_64_SSE);
+#endif
+ printf("\n4_64   ");
+ Bench4Helper(4,64,BenchConv4_64);
+#endif
+
+#ifdef USE_MMX
+ printf("\n8_4MMX");
+ Bench4Helper(8,4,BenchConv8_4_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n8_4SSE");
+ Bench4Helper(8,4,BenchConv8_4_SSE);
+#endif
+ printf("\n8_4   ");
+ Bench4Helper(8,4,BenchConv8_4);
+
+#ifdef USE_MMX
+ printf("\n8_16MMX");
+ Bench4Helper(8,16,BenchConv8_16_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n8_16SSE");
+ Bench4Helper(8,16,BenchConv8_16_SSE);
+#endif
+ printf("\n8_16   ");
+ Bench4Helper(8,16,BenchConv8_16);
+
+#ifdef USE_MMX
+ printf("\n8_32MMX");
+ Bench4Helper(8,32,BenchConv8_32_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n8_32SSE");
+ Bench4Helper(8,32,BenchConv8_32_SSE);
+#endif
+ printf("\n8_32   ");
+ Bench4Helper(8,32,BenchConv8_32);
+
+#if defined(uint64_t_defined)
+#if defined(USE_MMX)
+ printf("\n8_64MMX");
+ Bench4Helper(8,64,BenchConv8_64_MMX);
+#endif
+#if defined(USE_SSE)
+ printf("\n8_64SSE");
+ Bench4Helper(8,64,BenchConv8_64_SSE);
+#endif
+ printf("\n8_64   ");
+ Bench4Helper(8,64,BenchConv8_64);
+#endif
+
+#ifdef USE_MMX
+ printf("\n16_4MMX");
+ Bench4Helper(16,4,BenchConv16_4_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n16_4SSE");
+ Bench4Helper(16,4,BenchConv16_4_SSE);
+#endif
+ printf("\n16_4   ");
+ Bench4Helper(16,4,BenchConv16_4);
+
+#ifdef USE_MMX
+ printf("\n16_8MMX");
+ Bench4Helper(16,8,BenchConv16_8_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n16_8SSE");
+ Bench4Helper(16,8,BenchConv16_8_SSE);
+#endif
+ printf("\n16_8   ");
+ Bench4Helper(16,8,BenchConv16_8);
+
+#ifdef USE_MMX
+ printf("\n16_32MMX");
+ Bench4Helper(16,32,BenchConv16_32_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n16_32SSE");
+ Bench4Helper(16,32,BenchConv16_32_SSE);
+#endif
+ printf("\n16_32   ");
+ Bench4Helper(16,32,BenchConv16_32);
+
+#if defined(uint64_t_defined)
+#ifdef USE_MMX
+ printf("\n16_64MMX");
+ Bench4Helper(16,64,BenchConv16_64_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n16_64SSE");
+ Bench4Helper(16,64,BenchConv16_64_SSE);
+#endif
+ printf("\n16_64   ");
+ Bench4Helper(16,64,BenchConv16_64);
+#endif
+
+#ifdef USE_MMX
+ printf("\n32_16MMX");
+ Bench4Helper(32,16,BenchConv32_16_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n32_16SSE");
+ Bench4Helper(32,16,BenchConv32_16_SSE);
+#endif
+ printf("\n32_16   ");
+ Bench4Helper(32,16,BenchConv32_16);
+
+#if defined(uint64_t_defined)
+#ifdef USE_MMX
+ printf("\n32_64MMX");
+ Bench4Helper(32,64,BenchConv32_64_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n32_64SSE");
+ Bench4Helper(32,64,BenchConv32_64_SSE);
+#endif
+ printf("\n32_64   ");
+ Bench4Helper(32,64,BenchConv32_64);
+#endif
+
+ printf("\n\n");
+
+}
+
+
+
+void Bench5Helper(unsigned Planes1, unsigned Planes2, void (*Bench_Tester)(long i))
+{
+float f;
+ for(unsigned i=0; i<sizeof(vecSz5)/sizeof(unsigned); i++)
+ {
+   gRas1D_1 = CreateRaster1D(vecSz5[i], Planes1);	// Test both even and odd size.
+   gRas1D_2 = CreateRaster1D(vecSz5[i], Planes2);    
+   if(gRas1D_1!=NULL && gRas1D_2!=NULL)
+   {
+     f = 1000*AutoMeasure(Bench_Tester) / vecSz5[i];
+     printf("%8.5g ",f);
+   }
+   delete(gRas1D_1); gRas1D_1=NULL;
+   delete(gRas1D_2); gRas1D_2=NULL;
+ }
+}
+
+
+void Benchmark5(void)
+{
+unsigned i;
+
+ puts(" .............. Benchmarks #5..............");
+
+ printf("Conversions with acceleration [ns/px]:  ---> pixels\n   ");
+ for(i=0; i<sizeof(vecSz5)/sizeof(unsigned); i++)
+ {
+   printf(" %7d ",vecSz5[i]);	// print label
+ }
+ fflush(stdout);
+
+
+#ifdef USE_MMX
+ printf("\n4_8MMX");
+ Bench5Helper(4,8,BenchConv4_8_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n4_8SSE");
+ Bench5Helper(4,8,BenchConv4_8_SSE);
+#endif
+ printf("\n4_8   ");
+ Bench5Helper(4,8,BenchConv4_8);
+
+#ifdef USE_MMX
+ printf("\n4_16MMX");
+ Bench5Helper(4,16,BenchConv4_16_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n4_16SSE");
+ Bench5Helper(4,16,BenchConv4_16_SSE);
+#endif
+ printf("\n4_16   ");
+ Bench5Helper(4,16,BenchConv4_16);
+
+#ifdef USE_MMX
+ printf("\n4_32MMX");
+ Bench5Helper(4,32,BenchConv4_32_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n4_32SSE");
+ Bench5Helper(4,32,BenchConv4_32_SSE);
+#endif
+ printf("\n4_32   ");
+ Bench5Helper(4,32,BenchConv4_32);
+
+#if defined(uint64_t_defined)
+#ifdef USE_MMX
+ printf("\n4_64MMX");
+ Bench5Helper(4,64,BenchConv4_64_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n4_64SSE");
+ Bench5Helper(4,64,BenchConv4_64_SSE);
+#endif
+ printf("\n4_64   ");
+ Bench5Helper(4,64,BenchConv4_64);
+#endif
+
+#ifdef USE_MMX
+ printf("\n8_4MMX");
+ Bench5Helper(8,4,BenchConv8_4_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n8_4SSE");
+ Bench5Helper(8,4,BenchConv8_4_SSE);
+#endif
+ printf("\n8_4   ");
+ Bench5Helper(8,4,BenchConv8_4);
+
+#ifdef USE_MMX
+ printf("\n8_16MMX");
+ Bench5Helper(8,16,BenchConv8_16_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n8_16SSE");
+ Bench5Helper(8,16,BenchConv8_16_SSE);
+#endif
+ printf("\n8_16   ");
+ Bench5Helper(8,16,BenchConv8_16);
+
+#ifdef USE_MMX
+ printf("\n8_32MMX");
+ Bench5Helper(8,32,BenchConv8_32_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n8_32SSE");
+ Bench5Helper(8,32,BenchConv8_32_SSE);
+#endif
+ printf("\n8_32   ");
+ Bench5Helper(8,32,BenchConv8_32);
+
+#if defined(uint64_t_defined)
+#if defined(USE_MMX)
+ printf("\n8_64MMX");
+ Bench5Helper(8,64,BenchConv8_64_MMX);
+#endif
+#if defined(USE_SSE)
+ printf("\n8_64SSE");
+ Bench5Helper(8,64,BenchConv8_64_SSE);
+#endif
+ printf("\n8_64   ");
+ Bench5Helper(8,64,BenchConv8_64);
+#endif
+
+#ifdef USE_MMX
+ printf("\n16_4MMX");
+ Bench5Helper(16,4,BenchConv16_4_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n16_4SSE");
+ Bench5Helper(16,4,BenchConv16_4_SSE);
+#endif
+ printf("\n16_4   ");
+ Bench5Helper(16,4,BenchConv16_4);
+
+#ifdef USE_MMX
+ printf("\n16_8MMX");
+ Bench5Helper(16,8,BenchConv16_8_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n16_8SSE");
+ Bench5Helper(16,8,BenchConv16_8_SSE);
+#endif
+ printf("\n16_8   ");
+ Bench5Helper(16,8,BenchConv16_8);
+
+#ifdef USE_MMX
+ printf("\n16_32MMX");
+ Bench5Helper(16,32,BenchConv16_32_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n16_32SSE");
+ Bench5Helper(16,32,BenchConv16_32_SSE);
+#endif
+ printf("\n16_32   ");
+ Bench5Helper(16,32,BenchConv16_32);
+
+#if defined(uint64_t_defined)
+#ifdef USE_MMX
+ printf("\n16_64MMX");
+ Bench5Helper(16,64,BenchConv16_64_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n16_64SSE");
+ Bench5Helper(16,64,BenchConv16_64_SSE);
+#endif
+ printf("\n16_64   ");
+ Bench5Helper(16,64,BenchConv16_64);
+#endif
+
+#ifdef USE_MMX
+ printf("\n32_16MMX");
+ Bench5Helper(32,16,BenchConv32_16_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n32_16SSE");
+ Bench5Helper(32,16,BenchConv32_16_SSE);
+#endif
+ printf("\n32_16   ");
+ Bench5Helper(32,16,BenchConv32_16);
+
+#if defined(uint64_t_defined)
+#ifdef USE_MMX
+ printf("\n32_64MMX");
+ Bench5Helper(32,64,BenchConv32_64_MMX);
+#endif
+#ifdef USE_SSE
+ printf("\n32_64SSE");
+ Bench5Helper(32,64,BenchConv32_64_SSE);
+#endif
+ printf("\n32_64   ");
+ Bench5Helper(32,64,BenchConv32_64);
+#endif
+
+ printf("\n\n");
+
+}
\ No newline at end of file
