@@ -1,5 +1,5 @@
 /***************************************************************************
- * Unit:    raster            release 0.34                                 *
+ * Unit:    raster            release 0.37                                 *
  * Purpose: Prototypes for general manipulation n dimensional matrices     *
  * Modul:   raster.cc                                                      *
  * Licency: GPL or LGPL                                                    *
@@ -10,7 +10,7 @@
 
 #include "typedfs.h"
 
-#define RASTER_VERSION (0x100*0 | 34)
+#define RASTER_VERSION (0x100*0 | 37)
 
 #ifndef No_Memory
  #define No_Memory    0x1
@@ -64,14 +64,49 @@ typedef struct
 	} RGBQuad;
 #endif
 
+
+#if defined(LO_ENDIAN) || defined(HI_ENDIAN)
+/*
+ #ifdef LO_ENDIAN
+  #define Conv32_1(__Source,__Dest, __Count) Peel1BitNStep(__Source, 3+(const uint8_t*)(__Dest), __Count, 0x407)
+ #endif
+ #ifdef HI_ENDIAN
+  #define Conv32_1(__Source,__Dest, __Count) Peel1BitNStep(__Source, (const uint8_t*)(__Dest), __Count, 0x407)
+ #endif
+*/
+
+ #if defined(uint64_t_defined)
+  #ifdef LO_ENDIAN  
+   #define Conv64_1(__Source,__Dest, __Count) Peel1BitNStep(__Source, 7+(const uint8_t*)(__Dest), __Count, 0x807)
+   #define Conv64_8(__Source,__Dest, __Count) Peel8BitNStep(__Source, 7+(const uint8_t*)(__Dest), __Count, 8);
+  #endif
+  #ifdef HI_ENDIAN
+   #define Conv64_1(__Source,__Dest, __Count) Peel1BitNStep(__Source, (const uint8_t*)(__Dest), __Count, 0x807)
+   #define Conv64_8(__Source,__Dest, __Count) Peel8BitNStep(__Source, (const uint8_t*)(__Dest), __Count, 8);
+  #endif
+ #endif
+
+#endif
+
+#ifdef OPTIMISE_SPEED
+void Conv1_2(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+#endif
 void Conv1_4(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv1_8(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv1_16(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv1_24(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv1_32(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
+#ifdef OPTIMISE_SPEED
+void Conv2_1(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv2_8(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv2_16(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv2_24(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv2_32(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
+#endif
 void Conv4_1(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv4_8(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv4_16(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv4_24(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv4_32(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv8_1(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv8_4(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
@@ -83,10 +118,14 @@ void Conv16_4(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
 void Conv16_8(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
 void Conv16_24(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
 void Conv16_32(uint32_t *Dest, const uint16_t *Src, unsigned Size1D);
+void Conv24_1(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv24_4(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv24_8(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv24_16(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv24_32(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
-void Conv32_1(uint8_t *Dest, const uint32_t *Src, unsigned Size1D);
+#ifndef Conv32_1
+ void Conv32_1(uint8_t *Dest, const uint32_t *Src, unsigned Size1D);
+#endif
 void Conv32_4(uint8_t *Dest, const uint32_t *Src, unsigned Size1D);
 void Conv32_8(uint8_t *Dest, const uint32_t *Src, unsigned Size1D);
 void Conv32_16(uint16_t *Dest, const uint32_t *Src, unsigned Size1D);
@@ -94,6 +133,7 @@ void Conv32_24(uint8_t *Dest, const uint32_t *Src, unsigned Size1D);
 
 #if defined(uint64_t_defined)
 void Conv1_64(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv4_64(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv8_64(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
 void Conv16_64(uint64_t *Dest, const uint16_t *Src, unsigned Size1D);
 void Conv24_64(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
@@ -121,6 +161,82 @@ void Join1BitNStep(const uint8_t *Buffer1Bit, uint8_t *Buffer, unsigned count, u
 void Peel8BitNStep(uint8_t *Buffer8Bit, const uint8_t *BufferSrc, unsigned count, uint8_t ByteStep);
 void Join8BitNStep(const uint8_t *Buffer8Bit, uint8_t *Buffer, unsigned count, uint8_t ByteStep);
 
+
+#ifdef USE_MMX
+void Conv4_8_MMX(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv4_16_MMX(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv4_32_MMX(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv8_4_MMX(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv8_16_MMX(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv8_32_MMX(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv16_4_MMX(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
+void Conv16_8_MMX(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
+void Conv16_32_MMX(uint32_t *Dest, const uint16_t *Src, unsigned Size1D);
+void Conv32_16_MMX(uint16_t *Dest, const uint32_t *Src, unsigned Size1D);
+ #if defined(uint64_t_defined)
+  void Conv4_64_MMX(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
+  void Conv8_64_MMX(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
+  void Conv16_64_MMX(uint64_t *Dest, const uint16_t *Src, unsigned Size1D);
+  void Conv32_64_MMX(uint64_t *Dest, const uint32_t *Src, unsigned Size1D);
+ #endif
+#endif
+#ifdef USE_SSE
+void Conv4_8_SSE(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv4_16_SSE(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv4_32_SSE(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv8_4_SSE(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv8_16_SSE(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv8_32_SSE(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
+void Conv16_4_SSE(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
+void Conv16_8_SSE(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
+void Conv16_32_SSE(uint32_t *Dest, const uint16_t *Src, unsigned Size1D);
+void Conv32_16_SSE(uint16_t *Dest, const uint32_t *Src, unsigned Size1D);
+ #if defined(uint64_t_defined)
+  void Conv4_64_SSE(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
+  void Conv8_64_SSE(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
+  void Conv16_64_SSE(uint64_t *Dest, const uint16_t *Src, unsigned Size1D);
+  void Conv32_64_SSE(uint64_t *Dest, const uint32_t *Src, unsigned Size1D);
+ #endif
+#endif
+
+#if defined(USE_MMX) || defined(USE_SSE)
+extern void (*Conv4_8dyn)(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+extern void (*Conv4_16dyn)(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
+extern void (*Conv4_32dyn)(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
+extern void (*Conv8_4dyn)(uint8_t *Dest, const uint8_t *Src, unsigned Size1D);
+extern void (*Conv8_16dyn)(uint16_t *Dest, const uint8_t *Src, unsigned Size1D);
+extern void (*Conv8_32dyn)(uint32_t *Dest, const uint8_t *Src, unsigned Size1D);
+extern void (*Conv16_4dyn)(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
+extern void (*Conv16_8dyn)(uint8_t *Dest, const uint16_t *Src, unsigned Size1D);
+extern void (*Conv16_32dyn)(uint32_t *Dest, const uint16_t *Src, unsigned Size1D);
+extern void (*Conv32_16dyn)(uint16_t *Dest, const uint32_t *Src, unsigned Size1D);
+
+ #if defined(uint64_t_defined)
+  extern void (*Conv4_64dyn)(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
+  extern void (*Conv8_64dyn)(uint64_t *Dest, const uint8_t *Src, unsigned Size1D);
+  extern void (*Conv16_64dyn)(uint64_t *Dest, const uint16_t *Src, unsigned Size1D);
+  extern void (*Conv32_64dyn)(uint64_t *Dest, const uint32_t *Src, unsigned Size1D);    
+ #endif
+
+#define Conv4_8 Conv4_8dyn
+#define Conv4_16 Conv4_16dyn
+#define Conv4_32 Conv4_32dyn
+#define Conv4_64 Conv4_64dyn
+#define Conv8_4 Conv8_4dyn
+#define Conv8_16 Conv8_16dyn
+#define Conv8_32 Conv8_32dyn
+#define Conv8_64 Conv8_64dyn
+#define Conv16_4 Conv16_4dyn
+#define Conv16_8 Conv16_8dyn
+#define Conv16_32 Conv16_32dyn
+#define Conv16_64 Conv16_64dyn
+#define Conv32_16 Conv32_16dyn
+#define Conv32_64 Conv32_64dyn
+
+
+#endif
+
+
 #ifdef __cplusplus
 }
 
