@@ -1,5 +1,5 @@
 /*************************************************************
- * Unit:    raster            release 0.34                   *
+ * Unit:    raster            release 0.37                   *
  * Purpose: General manipulation n dimensional matrices      *
  *          n = 1, 2 and 3.			             *
  * Modul:   raster.cc                                        *
@@ -510,6 +510,8 @@ int i;
     case -1:
     case 1: Raster1DAbstract::Set(R1);
 	    return;
+    case 2: Conv2_1((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+	    return;
     case -4:
     case 4: Conv4_1((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
 	    return;
@@ -519,18 +521,13 @@ int i;
     case -16:
     case 16:Conv16_1((uint8_t *)Data1D,(const uint16_t *)R1.Data1D,Size1D);
 	    return;
+    case 24: Conv24_1((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+	    return;
     case 32:Conv32_1((uint8_t *)Data1D,(const uint32_t *)R1.Data1D,Size1D);
 	    return;
-#if defined(uint64_t_defined)
- #ifdef LO_ENDIAN
-    case 64:if(R1.Data1D)
-                Peel1BitNStep((uint8_t *)Data1D, 7+(const uint8_t*)R1.Data1D, Size1D, 0x807);
+#if defined(Conv64_1)
+    case 64:Conv64_1((uint8_t *)Data1D,(const uint64_t *)R1.Data1D,Size1D);
 	    return;
- #endif
- #ifdef HI_ENDIAN
-    case 64:Peel1BitNStep((uint8_t *)Data1D, (const uint8_t*)R1.Data1D, Size1D, 0x807);
-	    return;
- #endif
 #endif
     }
 
@@ -570,6 +567,8 @@ uint8_t Mask;
     {
     case 1: Raster1DAbstract::Get(R1);
 	    return;
+    case 2: Conv1_2((uint8_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
+	    return;
     case 4: Conv1_4((uint8_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
 	    return;
     case 8: Conv1_8((uint8_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
@@ -702,6 +701,40 @@ return 0;
 #endif
 
 
+#ifdef OPTIMISE_SPEED
+
+void Raster1D_2Bit::Get(Raster1DAbstract &R1) const
+{
+  switch(R1.GetPlanes())
+  {
+    case 1: Conv2_1((uint8_t*)R1.Data1D,(const uint8_t*)Data1D,Size1D);
+	    return;
+    case 8: Conv2_8((uint8_t*)R1.Data1D,(const uint8_t*)Data1D,Size1D);
+	    return;
+    case 16:Conv2_16((uint16_t*)R1.Data1D,(const uint8_t*)Data1D,Size1D);
+	    return;
+    case 24:Conv2_24((uint8_t*)R1.Data1D,(const uint8_t*)Data1D,Size1D);
+	    return;
+    case 32:Conv2_32((uint32_t*)R1.Data1D,(const uint8_t*)Data1D,Size1D);
+	    return;
+  }
+  Raster1DAbstract::Get(R1);
+}
+
+
+void Raster1D_2Bit::Set(const Raster1DAbstract &R1)
+{
+  switch(R1.GetPlanes())
+  {
+    case 1: Conv1_2((uint8_t*)Data1D,(const uint8_t*)R1.Data1D,Size1D);
+	    return;
+  }
+  Raster1DAbstract::Set(R1);
+}
+
+#endif
+
+
 /* ------------- 4 bit planes ------------- */
 
 void Raster1D_4Bit::SetValue1D(unsigned x, uint32_t NewValue)
@@ -764,8 +797,14 @@ void Raster1D_4Bit::Get(Raster1DAbstract &R1) const
 	    return;
     case 16:Conv4_16((uint16_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
 	    return;
+    case 24:Conv4_24((uint8_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
+	    return;
     case 32:Conv4_32((uint32_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
 	    return;
+#if defined(uint64_t_defined)
+    case 64:Conv4_64((uint64_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
+	    return;
+#endif
     }
 
   Raster1DAbstract::Get(R1);
@@ -784,6 +823,8 @@ void Raster1D_4Bit::Set(const Raster1DAbstract &R1)
 	    return;
     case 16:Conv16_4((uint8_t *)Data1D,(const uint16_t *)R1.Data1D,Size1D);
 	    return;
+    case 24: Conv24_4((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+	    return;
     case 32:Conv32_4((uint8_t *)Data1D,(const uint32_t *)R1.Data1D,Size1D);
 	    return;
     }
@@ -854,6 +895,10 @@ unsigned i;
     {
     case 1: Conv1_8((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
     	    return;
+#ifdef OPTIMISE_SPEED
+    case 2: Conv2_8((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+    	    return;
+#endif
     case 4: Conv4_8((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
     	    return;
     case 8: Raster1DAbstract::Set(R1);
@@ -864,18 +909,10 @@ unsigned i;
             return;
     case 32:Conv32_8((uint8_t *)Data1D,(const uint32_t *)R1.Data1D,Size1D);
 	    return;
-#if defined(uint64_t_defined)
- #ifdef LO_ENDIAN
-    case 64:if(R1.Data1D)
-                Peel8BitNStep((uint8_t *)Data1D, 7+(const uint8_t*)R1.Data1D, Size1D, 8);
+#if defined(uint64_t_defined) && defined(Conv64_8)
+    case 64:Conv64_8((uint8_t *)Data1D,(const uint64_t *)R1.Data1D,Size1D);
 	    return;
- #endif
- #ifdef HI_ENDIAN
-    case 64:Peel8BitNStep((uint8_t *)Data1D, (const uint8_t*)R1.Data1D, Size1D, 8);
-	    return;
- #endif
 #endif
-
     }
 
   uint8_t *N_Buffer8Bit = (uint8_t *)Data1D;
@@ -1076,6 +1113,10 @@ signed char shift;
     {
     case 1: Conv1_16((uint16_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
     	    return;
+#ifdef OPTIMISE_SPEED
+    case 2: Conv2_16((uint16_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+    	    return;
+#endif
     case 4: Conv4_16((uint16_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
             return;
     case 8: Conv8_16((uint16_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
@@ -1249,6 +1290,10 @@ void Raster1D_24Bit::Get(Raster1DAbstract &R1) const
 {  
   switch(R1.GetPlanes())
     {
+    case 1: Conv24_1((uint8_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
+	    return;
+    case 4: Conv24_4((uint8_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
+	    return;
     case 8: Conv24_8((uint8_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
 	    return;
     case 16:Conv24_16((uint16_t *)R1.Data1D,(const uint8_t *)Data1D,Size1D);
@@ -1270,6 +1315,10 @@ void Raster1D_24Bit::Set(const Raster1DAbstract &R1)
     {
     case 1: Conv1_24((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
 	    return;
+    case 2: Conv2_24((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+	    return;
+    case 4: Conv4_24((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+	    return;
     case 8: Conv8_24((uint8_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
 	    return;
     case 16:Conv16_24((uint8_t *)Data1D,(const uint16_t *)R1.Data1D,Size1D);
@@ -1408,7 +1457,6 @@ signed char shift;
     case 64:Conv32_64((uint64_t *)R1.Data1D,(const uint32_t*)Data1D,Size1D);
 	    return;
 #endif
-
     }
 
   const uint32_t *N_Buffer32Bit = (const uint32_t *)Data1D;
@@ -1440,6 +1488,10 @@ signed char shift;
     {
     case 1: Conv1_32((uint32_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
     	    return;
+#ifdef OPTIMISE_SPEED
+    case 2: Conv2_32((uint32_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+    	    return;
+#endif
     case 4: Conv4_32((uint32_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
 	    return;
     case 8: Conv8_32((uint32_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
@@ -1699,23 +1751,15 @@ void Raster1D_64Bit::Join8Bit(const void *Buffer8Bit, uint8_t plane8)
 void Raster1D_64Bit::Get(Raster1DAbstract &R1) const
 {
   switch(R1.GetPlanes())
-    {
-#ifdef LO_ENDIAN
-    case 1: if(Data1D)
-                Peel1BitNStep((uint8_t *)R1.Data1D, 7+(const uint8_t*)Data1D, Size1D, 0x807);
-	    return;
-    case 8: if(Data1D)
-                Peel8BitNStep((uint8_t *)R1.Data1D, 7+(const uint8_t*)Data1D, Size1D, 8);
+  {
+#ifdef Conv64_1
+    case 1: Conv64_1((uint8_t *)R1.Data1D,(const uint64_t *)Data1D,Size1D);
 	    return;
 #endif
-#ifdef HI_ENDIAN
-    case 1: Peel1BitNStep((uint8_t *)R1.Data1D, (const uint8_t*)Data1D, Size1D, 0x807);
-	    return;
-    case 8: Peel8BitNStep((uint8_t *)R1.Data1D, (const uint8_t*)Data1D, Size1D, 8);
+#ifdef Conv64_8
+    case 8: Conv64_8((uint8_t *)R1.Data1D,(const uint64_t *)Data1D,Size1D);
 	    return;
 #endif
-    //case 8: Conv64_8((uint8_t *)R1.Data1D,(const uint64_t *)Data1D,Size1D);
-    //	    return;
     case 16:Conv64_16((uint16_t *)R1.Data1D,(const uint64_t *)Data1D,Size1D);
 	    return;
     case 24:Conv64_24((uint8_t *)R1.Data1D,(const uint64_t *)Data1D,Size1D);
@@ -1733,6 +1777,8 @@ void Raster1D_64Bit::Set(const Raster1DAbstract &R1)
     {
     case 1: Conv1_64((uint64_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
 	    return;
+    case 4: Conv4_64((uint64_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
+	    return;
     case 8: Conv8_64((uint64_t *)Data1D,(const uint8_t *)R1.Data1D,Size1D);
 	    return;
     case 16:Conv16_64((uint64_t *)Data1D,(const uint16_t *)R1.Data1D,Size1D);
@@ -1814,7 +1860,6 @@ void Raster1D_32FltBit::Get(Raster1DAbstract &R1) const
 {
 const float *N_BufferFlt = (float *)Data1D;
 int i;
-float MaxVal;
 
   i = R1.GetPlanes();
   
@@ -1830,7 +1875,7 @@ float MaxVal;
     }
   else
     {
-    MaxVal = (1<<i) - 1;
+    const float MaxVal = (1<<i) - 1;
 
     for(i=0;i<Size1D;i++)
       {
@@ -1851,7 +1896,6 @@ void Raster1D_32FltBit::Set(const Raster1DAbstract &R1)
 {
 float *N_BufferFlt = (float *)Data1D;
 int i;
-float MaxVal;
 
   i = R1.GetPlanes();
   
@@ -1867,7 +1911,7 @@ float MaxVal;
     }
   else
     {
-    MaxVal = (1<<i) - 1;
+    const float MaxVal = (1<<i) - 1;
     for(i=0; i<Size1D; i++)
       {
       *N_BufferFlt++ = R1.GetValue1D(i) * MaxVal;
@@ -1945,10 +1989,8 @@ void Raster1D_64FltBit::PTR_SetValue1Dd(void *RAW_Data1D, unsigned x, double New
 void Raster1D_64FltBit::Get(Raster1DAbstract &R1) const
 {
 const double *N_BufferFlt = (const double *)Data1D;
-int i;
-float MaxVal;
 
-  i = R1.GetPlanes();
+  const int i = R1.GetPlanes();
   
   if(i<0)		// Float or double.
     {
@@ -1956,22 +1998,22 @@ float MaxVal;
       memcpy(R1.Data1D, Data1D, Size1D*sizeof(float));
     else
       {
-      for(i=0; i<Size1D; i++)
-        R1.SetValue1Dd(i, *N_BufferFlt++);
+      for(unsigned x=0; x<Size1D; x++)
+        R1.SetValue1Dd(x, *N_BufferFlt++);
       }
     }
   else
     {
-    MaxVal = (1<<i) - 1;
+    const float MaxVal = (1<<i) - 1;
 
-    for(i=0;i<Size1D;i++)
+    for(unsigned x=0; x<Size1D; x++)
       {
       if(*N_BufferFlt>1)
-        R1.SetValue1D(i,MaxVal);
+        R1.SetValue1D(x,MaxVal);
       else if(*N_BufferFlt<0)
-        R1.SetValue1D(i,0);
+        R1.SetValue1D(x,0);
       else
-        R1.SetValue1D(i,*N_BufferFlt*MaxVal);
+        R1.SetValue1D(x,*N_BufferFlt*MaxVal);
 
       N_BufferFlt++;
       }
@@ -1982,10 +2024,8 @@ float MaxVal;
 void Raster1D_64FltBit::Set(const Raster1DAbstract &R1)
 {
 double *N_BufferFlt = (double *)Data1D;
-int i;
-double MaxVal;
 
-  i = R1.GetPlanes();
+  const int i = R1.GetPlanes();
   
   if(i<0)		// Float or double.
     {
@@ -1993,16 +2033,16 @@ double MaxVal;
       memcpy(Data1D, R1.Data1D, Size1D*sizeof(float));
     else
       {
-      for(i=0; i<Size1D; i++)
-        *N_BufferFlt++ = R1.GetValue1Dd(i);
+      for(unsigned x=0; x<Size1D; x++)
+        *N_BufferFlt++ = R1.GetValue1Dd(x);
       }
     }
   else
     {
-    MaxVal = (1<<i) - 1;
-    for(i=0;i<Size1D;i++)
+    const double MaxVal = (1<<i) - 1;
+    for(unsigned x=0; x<Size1D; x++)
       {
-      *N_BufferFlt++ = R1.GetValue1D(i) * MaxVal;
+      *N_BufferFlt++ = R1.GetValue1D(x) * MaxVal;
       }
     }
 }
@@ -2454,7 +2494,6 @@ void Raster1D_8BitRGB::Set(const Raster1DAbstract &R1)
 
 #ifdef OPTIMISE_SPEED
 void  Raster1D_8BitRGB::Set(unsigned x, const RGBQuad *RGB)
-
 {
   if(Size1D<=x || RGB==NULL) return;
   x *= 3;
