@@ -1,5 +1,5 @@
 /*********************************************************
-* unit:    raster            release 0.33                *
+* unit:    raster            release 0.37                *
 * purpose: general manipulation n dimensional matrices   *
 *          n = 1, 2 and 3.			         *
 * licency:     GPL or LGPL                               *
@@ -147,6 +147,13 @@ uint8_t Mask;
 #endif
 
 
+/*--------------2 bit---------------- */
+
+
+// Located in rasterut.cc
+// void Conv2_1(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
+
+
 /*--------------4 bit---------------- */
 
 
@@ -187,16 +194,31 @@ void Conv4_8(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
 
 void Conv4_16(uint16_t *Dest, const uint8_t *Src, unsigned Size1D)
 {
-static const uint16_t Conv4_16TAB[16] = { 
-       0x0000, 0x1111, 0x2222, 0x3333, 0x4444, 0x5555, 0x6666, 0x7777,
-       0x8888, 0x9999, 0xAAAA, 0xBBBB, 0xCCCC, 0xDDDD, 0xEEEE, 0xFFFF};
+//static const uint16_t Conv4_16TAB[16] = { 
+//       0x0000, 0x1111, 0x2222, 0x3333, 0x4444, 0x5555, 0x6666, 0x7777,
+//       0x8888, 0x9999, 0xAAAA, 0xBBBB, 0xCCCC, 0xDDDD, 0xEEEE, 0xFFFF};
+  while(Size1D>0)
+  {
+    //*Dest++ = Conv4_16TAB[*Src>>4];
+    *Dest++ = 0x1111 * (*Src>>4);
+    if(Size1D-- <= 1) break;
+    //*Dest++ = Conv4_16TAB[*Src & 0xF];
+    *Dest++ = 0x1111 * (*Src & 0xF);
+    Src++;
+    Size1D--;
+  }
+}
+
+
+void Conv4_24(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
   while(Size1D>0)
     {
-    *Dest = Conv4_16TAB[*Src>>4];
-    Dest++;
+    Dest[0] = Dest[1] = Dest[2] = (*Src>>4) | (*Src&0xF0);  /* Duplicate nibble. */
+    Dest+=3;
     if(Size1D-- <= 1) break;
-    *Dest = Conv4_16TAB[*Src & 0xF];
-    Dest++;
+    Dest[0] = Dest[1] = Dest[2] = (uint16_t)(*Src & 0xF) | (*Src<<4);
+    Dest+=3;
     Src++;
     Size1D--;
     }
@@ -205,22 +227,53 @@ static const uint16_t Conv4_16TAB[16] = {
 
 void Conv4_32(uint32_t *Dest, const uint8_t *Src, unsigned Size1D)
 {
-static const uint32_t Conv4_32TAB[16] = { 
-       0x00000000, 0x11111111, 0x22222222, 0x33333333, 0x44444444, 0x55555555, 0x66666666, 0x77777777,
-       0x88888888, 0x99999999, 0xAAAAAAAA, 0xBBBBBBBB, 0xCCCCCCCC, 0xDDDDDDDD, 0xEEEEEEEE, 0xFFFFFFFF};
+//static const uint32_t Conv4_32TAB[16] = { 
+//       0x00000000, 0x11111111, 0x22222222, 0x33333333, 0x44444444, 0x55555555, 0x66666666, 0x77777777,
+//       0x88888888, 0x99999999, 0xAAAAAAAA, 0xBBBBBBBB, 0xCCCCCCCC, 0xDDDDDDDD, 0xEEEEEEEE, 0xFFFFFFFF};
   while(Size1D>0)
-    {
-    *Dest = Conv4_32TAB[*Src>>4];
-    Dest++;
+  {
+    //*Dest++ = Conv4_32TAB[*Src>>4];
+    *Dest++ = 0x11111111 * (*Src>>4);		// 32 bit multiplication is faster than table access
     if(Size1D-- <= 1) break;
-    *Dest = Conv4_32TAB[*Src & 0xF];
-    Dest++;
+    //*Dest++ = Conv4_32TAB[*Src & 0xF];
+    *Dest++ = 0x11111111 * (*Src & 0xF);
     Src++;
     Size1D--;
-    }
+  }
 }
 
 
+#if defined(uint64_t_defined)
+
+void Conv4_64(uint64_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+#if !defined(POINTER_SIZE) || POINTER_SIZE<64
+static const uint64_t Conv4_64TAB[16] = { 
+	0x0000000000000000, 0x1111111111111111, 0x2222222222222222, 0x3333333333333333, 0x4444444444444444,
+	0x5555555555555555, 0x6666666666666666, 0x7777777777777777, 0x8888888888888888, 0x9999999999999999,
+	0xAAAAAAAAAAAAAAAA, 0xBBBBBBBBBBBBBBBB, 0xCCCCCCCCCCCCCCCC, 0xDDDDDDDDDDDDDDDD, 0xEEEEEEEEEEEEEEEE, 0xFFFFFFFFFFFFFFFF};
+#endif
+  while(Size1D>0)
+  {
+#if !defined(POINTER_SIZE) || POINTER_SIZE<64
+    *Dest++ = Conv4_64TAB[*Src>>4];
+#else
+    *Dest++ = 0x1111111111111111 * (*Src>>4);		// 64 bit multiplication is slower than table access on 32 bit platforms
+#endif
+    if(Size1D-- <= 1) break;
+#if !defined(POINTER_SIZE) || POINTER_SIZE<64
+    *Dest++ = Conv4_64TAB[*Src & 0xF];
+#else
+    *Dest++ = 0x1111111111111111 * (*Src & 0xF);
+#endif
+    Src++;
+    Size1D--;
+  }
+}
+
+#endif
+
+
 /*---------- 8bit = 1byte ------------ */
 
 void Conv8_1(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
@@ -377,13 +430,48 @@ void Conv16_64(uint64_t *Dest, const uint16_t *Src, unsigned Size1D)
 /*--------------------------------- */
 
 
+void Conv24_1(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+uint8_t Mask;
+
+  Src+=2;
+  while(Size1D>0)
+    {
+    *Dest = 0;
+    for(Mask=0x80; Mask!=0; Mask>>=1)
+      {
+      if(*Src>=0x80) *Dest |= Mask;
+      if(--Size1D<=0) return;
+      Src+=3;
+      }
+    Dest++;
+    }
+}
+
+
+void Conv24_4(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+  Src += 2;
+  while(Size1D>0)
+    {
+    *Dest = 0xF0 & (uint8_t)(*Src);
+    Src += 3;
+    if(Size1D-- <= 1) break;
+    *Dest |= (uint8_t)(*Src>>4);
+    Src += 3;
+    Dest++;    
+    Size1D--;
+    }
+}
+
+
 void Conv24_8(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
 {
   Src += 2;		/* point to 3rd byte.*/
   while(Size1D-->0)
   {    
     *Dest++ = *Src;
-    Src += 3;    
+    Src += 3;
   }
 }
 
@@ -402,8 +490,15 @@ void Conv24_16(uint16_t *Dest, const uint8_t *Src, unsigned Size1D)
 void Conv24_32(uint32_t *Dest, const uint8_t *Src, unsigned Size1D)
 {
   while(Size1D-->0)
-  {    
-    *Dest++ = Src[0]*0x0100 | Src[1]*0x00010000 | Src[2]*0x01000001;
+  {
+#ifdef __MSDOS__
+    *(uint8_t*)Dest = ((uint8_t*)Dest)[3] = Src[2];
+    ((uint8_t*)Dest)[2] = Src[1];
+    ((uint8_t*)Dest)[1] = Src[0];
+    Dest++;
+#else
+    *Dest++ = Src[0]*(uint32_t)0x0100 | Src[1]*(uint32_t)0x00010000 | (uint32_t)Src[2]*0x01000001;
+#endif
     Src += 3;
   }
 }
@@ -424,6 +519,7 @@ void Conv24_64(uint64_t *Dest, const uint8_t *Src, unsigned Size1D)
 /*--------------------------------- */
 
 
+#ifndef Conv32_1
 void Conv32_1(uint8_t *Dest, const uint32_t *Src, unsigned Size1D)
 {
 uint8_t Mask;
@@ -433,12 +529,13 @@ uint8_t Mask;
     *Dest = 0;
     for(Mask=0x80; Mask!=0; Mask>>=1)
       {
-      if(*Src++>=0x80000000) *Dest |= Mask;
+      if(*Src++>=0x80000000) *Dest |= Mask;	/* Must read DWORD to be endian invariant */
       if(--Size1D<=0) return;
       }
     Dest++;
     }
 }
+#endif
 
 
 void Conv32_4(uint8_t *Dest, const uint32_t *Src, unsigned Size1D)
