@@ -1,9 +1,9 @@
 /******************************************************************************
- * program:     rasimg library 0.32                                           *
+ * program:     rasimg library 0.37                                           *
  * function:    Miscellaneous utilities for manipulation with images.         *
  * modul:       rasterut.cc                                                   *
  * licency:     GPL or LGPL                                                   *
- * Copyright: (c) 1998-2024 Jaroslav Fojtik                                   *
+ * Copyright: (c) 1998-2025 Jaroslav Fojtik                                   *
  ******************************************************************************/
 #include<stdio.h>
 #include<string.h>
@@ -55,6 +55,152 @@ return 0;
 }
 
 
+////////////////////////////////////////////////////////////////////
+
+
+#ifdef OPTIMISE_SPEED
+
+void Conv1_2(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+uint8_t Mask1;
+uint8_t Mask2;
+
+  while(Size1D>0)
+  {
+    Mask1 = 0x80;   
+    do
+      {
+      Mask2 = 0xC0;
+      if((*Src & Mask1) == 0)
+         *Dest = 0;
+      else
+        *Dest = Mask2;
+       
+      if(--Size1D<=0) return;
+      Mask1 >>= 1;
+      Mask2 >>= 2;
+      if((*Src & Mask1) != 0) *Dest|=Mask2;
+
+      if(--Size1D<=0) return;
+      Mask1 >>= 1;
+      Mask2 >>= 2;
+      if((*Src & Mask1) != 0) *Dest|=Mask2;
+
+      if(--Size1D<=0) return;
+      Mask1 >>= 1;
+      Mask2 >>= 2;
+      if((*Src & Mask1) != 0) *Dest|=Mask2;
+
+      Dest++;
+      if(--Size1D<=0) return;
+      Mask1 >>= 1;
+      } while(Mask1!=0);
+    Src++;
+  }
+}
+
+
+void Conv2_1(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+uint8_t Mask;
+
+  while(Size1D>0)
+  {
+    *Dest = 0;
+    for(Mask=0x80; Mask!=0; Mask>>=1)
+    {
+      if(*Src>=0x80) *Dest|=Mask;
+      if(--Size1D<=0) return;
+      Mask >>= 1;
+      if((0x3F & *Src)>=0x20) *Dest|=Mask;
+      if(--Size1D<=0) return;
+      Mask >>= 1;
+      if((0xF & *Src)>=0x08) *Dest |= Mask;
+      if(--Size1D<=0) return;
+      Mask >>= 1;
+      if((0x3 & *Src++)>=0x02) *Dest |= Mask;
+      if(--Size1D<=0) return;
+    }
+    Dest++;
+  }
+}
+
+
+void Conv2_8(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+uint8_t Shift;
+
+  while(Size1D>0)
+  {
+    Shift = 8;
+    do
+    {
+      Shift -= 2;
+      *Dest++ = ((*Src >> Shift) & 3) * 0x55;
+      if(--Size1D<=0) return;
+    } while(Shift>0);
+    Src++;
+  }
+}
+
+
+void Conv2_16(uint16_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+uint8_t Shift;
+
+  while(Size1D>0)
+  {
+    Shift = 8;
+    do
+    {
+      Shift -= 2;
+      *Dest++ = ((*Src >> Shift) & 3) * 0x5555;
+      if(--Size1D<=0) return;
+    } while(Shift>0);
+    Src++;
+  }
+}
+
+
+void Conv2_24(uint8_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+uint8_t Shift;
+
+  while(Size1D>0)
+  {
+    Shift = 8;
+    do
+    {
+      Shift -= 2;
+      Dest[0] = Dest[1] = Dest[2] = ((*Src >> Shift) & 3) * 0x55;
+      Dest+=3;
+      if(--Size1D<=0) return;
+    } while(Shift>0);
+    Src++;
+  }
+}
+
+
+void Conv2_32(uint32_t *Dest, const uint8_t *Src, unsigned Size1D)
+{
+uint8_t Shift;
+
+  while(Size1D>0)
+  {
+    Shift = 8;
+    do
+    {
+      Shift -= 2;
+      *Dest++ = ((*Src >> Shift) & 3) * 0x55555555;
+      if(--Size1D<=0) return;
+    } while(Shift>0);
+    Src++;
+  }
+}
+
+#endif
+
+
 #if defined(uint64_t_defined)
 
 /*
@@ -118,7 +264,7 @@ void Flip1D(Raster1DAbstract *r1D)
     case 32: Flip32((uint32_t*)r1D->GetRow(), r1D->GetSize1D());
 	     return;
     case 64: Flip64((uint64_t_fix*)r1D->GetRow(), r1D->GetSize1D());
-	     return;    
+	     return;
   }
   FlipN((uint8_t*)r1D->GetRow(), r1D->GetSize1D(),p);
 }
@@ -199,10 +345,8 @@ unsigned y;
 	       break;
       case 32: Flip32((uint32_t*)r2D->GetRow(y), Size1D);
 	       break;
-#if defined(uint64_t_defined)
-      case 64: Flip64((uint64_t*)r2D->GetRow(y), Size1D);
+      case 64: Flip64((uint64_t_fix*)r2D->GetRow(y), Size1D);
 	       break;
-#endif
 /*
       default: {
                  unsigned x1 = 0;
@@ -275,10 +419,8 @@ unsigned y, z;
 	       break;
       case 32: Flip32((uint32_t*)Row2D[y], Size1D);
 	       break;
-#if defined(uint64_t_defined)
-      case 64: Flip64((uint64_t*)Row2D[y], Size1D);
+      case 64: Flip64((uint64_t_fix*)Row2D[y], Size1D);
 	       break;
-#endif
 /*
       default: unsigned x1 = 0;
 	       unsigned x2 = r3D->Size1D - 1;
@@ -438,3 +580,4 @@ unsigned char BPP2;
   }
 }
 
+
